# -*- encoding: UTF8 -*-
# Test harness for CreateSwarmReview.py

from __future__ import print_function

import sys
import unittest
import os
import re

sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)))
import testp4
import P4
import logging
import subprocess

from CreateSwarmReview import CreateSwarmReview

os.environ["LOGS"] = "."
LOGGER_NAME = "TestCreateSwarmReview"
LOG_FILE = "log-CreateSwarmReview.log"

python3 = sys.version_info[0] >= 3

def ensureDirectory(directory):
    if not os.path.isdir(directory):
        os.makedirs(directory)

def localDirectory(root, *dirs):
    "Create and ensure it exists"
    dir_path = os.path.join(root, *dirs)
    ensureDirectory(dir_path)
    return dir_path

def create_file(file_name, contents):
    "Create file with specified contents"
    ensureDirectory(os.path.dirname(file_name))
    if python3:
        contents = bytes(contents.encode())
    with open(file_name, 'wb') as f:
        f.write(contents)

def append_to_file(file_name, contents):
    "Append contents to file"
    if python3:
        contents = bytes(contents.encode())
    with open(file_name, 'ab+') as f:
        f.write(contents)

class TestCreateSwarmReview(unittest.TestCase):
    def __init__(self, methodName='runTest'):
        super(TestCreateSwarmReview, self).__init__(methodName=methodName)
        self.logger = logging.getLogger(LOGGER_NAME)
        self.logger.setLevel(logging.DEBUG)
        logformat = '%(levelname)s [%(asctime)s] [%(filename)s : %(lineno)d] - %(message)s'
        logging.basicConfig(format=logformat, filename=LOG_FILE, level=logging.DEBUG)

    def assertRegex(self, *args, **kwargs):
        if python3:
            return super(TestCreateSwarmReview, self).assertRegex(*args, **kwargs)
        else:
            return super(TestCreateSwarmReview, self).assertRegexpMatches(*args, **kwargs)

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testCreateSwarmReview(self):
        """trigger fires and sends expected info to Swarm"""
        self.server = testp4.P4Server()
        trigpath = os.path.join(os.path.dirname(os.path.abspath(__file__)), "CreateSwarmReview.py")
        p4 = self.server.p4
        p4.logger = self.logger
        # This works if no spaces in server root pathname!
        port = p4.port.replace('"', '')
        self.logger.debug("port: |%s|" % port)
        triggers = p4.fetch_triggers()
        triggers['Triggers'] = ['create_swarm_review change-commit //... " python ' + trigpath +
                                " -p %quote%" + port + "%quote% -u " + p4.user +
                               '  %change% "']
        self.logger.debug(triggers)
        p4.save_triggers(triggers)
        # Reconnect to pick up changes
        p4.disconnect()
        p4.connect()

        inside = localDirectory(self.server.client_root, "inside")
        inside_file1 = os.path.join(inside, "inside_file1")
        create_file(inside_file1, 'Test content')

        p4.run('add', inside_file1)
        p4.run('submit', '-d', 'inside_file1 added')


if __name__ == '__main__':
    unittest.main()
